//Sketch for the Digital DataPort Logics from Printed-Droid.com
//V2.0

#include <FastLED.h>

// Serial Commands (\n is command seperator!)
// -> L0 disable Large Logic
// -> L1 Large Logic Red bar
// -> L2 Large Logic Rainbow
// -> C1 CBI Red Random Matrix
// -> C2 CBI Rainbow
// -> C3 CBI Rainbow with Sparkels

//TODO 
//     -> CBI Effekte (herz, smiley,...)
//     -> VU-Meter via external sound
//     -> VU-Fake Effekte (bottom-up, middle,...)
//     -> I2C Ansteuerung für Marcduino!

#define LeftDoorPin 16 //  Used for door open/closed signals on the Arduino Mini Breakout using 10K pulldown resistors
#define RightDoorPin 15 //  Used for door open/closed signals on the Arduino Mini Breakout using 10K pulldown resistors

#define monitorVCC

//Set this to which Analog Pin you use for the voltage in.
#define analoginput 0

#define greenVCC 12.5    // Green LED on if above this voltage
#define yellowVCC 12.0   // Yellow LED on if above this voltage
#define redVCC 11.5      // Red LED on if above this voltage

// For 12-15volts: R1=47k, R2=24k
// For 24volts: R1=30k, R2=6.2k
// For 30volts: R1=47k, R2=9.4k
#define R1 47000.0     // >> resistance of R1 in ohms << the more accurate these values are
#define R2 24000.0     // >> resistance of R2 in ohms << the more accurate the measurement will be

float vout = 0.0;       // for voltage out measured analog input
int value = 0;          // used to hold the analog value coming out of the voltage divider
float vin = 0.0;        // voltage calulcated... since the divider allows for 15 volts


#define LED_TYPE    WS2811
#define COLOR_ORDER GRB
#define BRIGHTNESS 50

//Bottom 6 LEDS 1-6
//2 Big right side
#define DATA_PINA   2
#define NUM_LEDSA   8
CRGB ledsa[NUM_LEDSA];

//vu-meter leds
#define DATA_PINB   3
#define NUM_LEDSB    28
CRGB ledsb[NUM_LEDSB];

//right top leds
// 6 rows 4 cols
#define DATA_PINC  10
#define NUM_LEDSC  24
CRGB ledsc[NUM_LEDSC];

//Top left leds
#define DATA_PIND  11
#define NUM_LEDSD  18
CRGB ledsd[NUM_LEDSD];

//Large Logic Display
#define DATA_PINE 7
#define NUM_LEDSE 43
CRGB ledse[NUM_LEDSE];

//CBI Leds
#define DATA_PINF 6
#define NUM_LEDSF 23
CRGB ledsf[NUM_LEDSF];

uint8_t effektcbi=0;
uint8_t gHue = 0;

#define CBISPEED_RANDOM 200

//Large Logic Panel
#define LARGELOGICSPEED 15
#define LARGELOGICWAIT 1000
int llcnt=5;
int llindex=0-llcnt;
int llstep=1;
int llwait=0;

int lleffekt=0;

#define BLUELEDSPEED 200

#define RIGHTVUSPEED 200
static int bar=0;

#define REDBOTTOMLEDSPEED 200

#define BOTTOMLEDSPEED 200

#define TOPLEDSPEED 200

String inputString = "";
bool stringComplete = false;

void setup() {

  Serial.begin(57600);
    
  delay(2000); // 2 second delay for recovery
  FastLED.addLeds<LED_TYPE,DATA_PINA,COLOR_ORDER>(ledsa, NUM_LEDSA).setCorrection(TypicalLEDStrip);
  FastLED.addLeds<LED_TYPE,DATA_PINB,COLOR_ORDER>(ledsb, NUM_LEDSB).setCorrection(TypicalLEDStrip);
  FastLED.addLeds<LED_TYPE,DATA_PINC,COLOR_ORDER>(ledsc, NUM_LEDSC).setCorrection(TypicalLEDStrip);
  FastLED.addLeds<LED_TYPE,DATA_PIND,COLOR_ORDER>(ledsd, NUM_LEDSD).setCorrection(TypicalLEDStrip);
  FastLED.addLeds<LED_TYPE,DATA_PINE,COLOR_ORDER>(ledse, NUM_LEDSE).setCorrection(TypicalLEDStrip);
  FastLED.addLeds<LED_TYPE,DATA_PINF,COLOR_ORDER>(ledsf, NUM_LEDSF).setCorrection(TypicalLEDStrip);
  FastLED.setBrightness(BRIGHTNESS);
  
  for (int i=0;i<NUM_LEDSA;i++){ ledsa[i] = CRGB::Black; }
  for (int i=0;i<NUM_LEDSB;i++){ ledsb[i] = CRGB::Black; }
  for (int i=0;i<NUM_LEDSC;i++){ ledsc[i] = CRGB::Black; }
  for (int i=0;i<NUM_LEDSD;i++){ ledsd[i] = CRGB::Black; }
  for (int i=0;i<NUM_LEDSE;i++){ ledse[i] = CRGB::Black; }
  for (int i=0;i<NUM_LEDSF;i++){ ledsf[i] = CRGB::Black; }

  // Pin on the Arduino Mini Breakout Board connected to left or right door 
  // switch HIGH=Door closed (NC when door closed)  
  pinMode(LeftDoorPin, INPUT_PULLUP);
  pinMode(RightDoorPin, INPUT_PULLUP);

}

void loop() {

  //Serial Command arrived!
  if (stringComplete) {
    Serial.println("Command:");
    Serial.println(inputString);

    if (inputString.equals("C1")){
      effektcbi=0;
    }
    if (inputString.equals("C2")){
      effektcbi=1;
    }
    if (inputString.equals("C3")){
      effektcbi=2;
    }

    if (inputString.equals("L0")){
      lleffekt=0;
    }
    if (inputString.equals("L1")){
      lleffekt=1;
    }
    if (inputString.equals("L2")){
      lleffekt=2;
    }
    
    // clear the string:
    inputString = "";
    stringComplete = false;

  }
  
  // Open(Low)/Closed(High) Status of Dataport Door
  int LeftDoorStatus = digitalRead(LeftDoorPin);
  // Open(Low)/Closed(High) Status of Charge Bay Indicator Door
  int RightDoorStatus = digitalRead(RightDoorPin);

  if (LeftDoorStatus == LOW) { //DPL Door is open!
    bluecol();
    rightvu();
    red_big();
    white_bottom();
    top_yellow_green();
    vu();
  }
  else { //Shutdown DPL LEDS if door is closed!
    for (int i=0;i<NUM_LEDSA;i++){
      ledsa[i] = CRGB::Black;
    }
    for (int i=0;i<NUM_LEDSB;i++){
      ledsb[i] = CRGB::Black;
    }
    for (int i=0;i<NUM_LEDSC;i++){
      ledsc[i] = CRGB::Black;
    }
    for (int i=0;i<NUM_LEDSD;i++){
      ledsd[i] = CRGB::Black;
    }
  }

  if (RightDoorStatus == LOW) { //CBI Door is open!
     updateCBILEDs();
     #ifdef monitorVCC
       getVCC();
     #endif    
  }
  else {
    //switch off all CBI leds
    for (int i=0;i<NUM_LEDSF;i++){ //smd leds
      ledsf[i] = CRGB::Black;
    }
  }

  if (lleffekt==1){
     largelogic();
  }
  else if (lleffekt==2){
     fill_rainbow( ledse, 43, gHue, 7);
  }
  else {
    for (int i=0;i<NUM_LEDSE;i++){ ledse[i] = CRGB::Black; }
  }
  
  FastLED.show();
  EVERY_N_SECONDS( 10 ) {
    //TODO  
  }
  EVERY_N_MILLISECONDS( 20 ) { gHue++; }

}

void serialEvent() {
  while(Serial.available()){
    char inChar = (char)Serial.read();
    if (inChar == '\n') {
      stringComplete = true;
    }
    else {
      inputString += inChar;
    }
  }
}

void largelogic() {
  static unsigned long timeLast=0;
  unsigned long elapsed=millis();
  if ((elapsed - timeLast) < LARGELOGICSPEED) return;

  if (llwait==2){
    for (int i=0;i<NUM_LEDSE;i++){
      ledse[i] = CRGB::Black;
    }
    
    if ((elapsed - timeLast) < LARGELOGICWAIT){
      return;
    }
    llwait=0;
  }
  
  timeLast = elapsed;

  for (int i=0;i<NUM_LEDSE;i++){
    ledse[i] = CRGB::Black;
  }

  for ( int i=0; i<llcnt;i++){
    ledse[llindex+i] = CRGB::Red;  
  }

  llindex+=llstep;

  if (llindex==(NUM_LEDSE-llcnt) || llindex== (0-llcnt)){
    llstep = - llstep;
    llwait++;
  }

}

void bluecol() {

  static unsigned long timeLast=0;
  static unsigned long variabledelay=BLUELEDSPEED;
  unsigned long elapsed=millis();
  if ((elapsed - timeLast) < variabledelay) return;
  timeLast = elapsed;  
  variabledelay=random(10, BLUELEDSPEED);
  
  //random ersten 4 Bits
  //B00111111
  int mask = random(0,64);

  ledsc[0]=CRGB::Black;
  ledsc[4]=CRGB::Black;
  ledsc[8]=CRGB::Black;
  ledsc[12]=CRGB::Black;
  ledsc[16]=CRGB::Black;
  ledsc[20]=CRGB::Black;

  if (bitRead(mask,5)){ ledsc[0]=CRGB::Blue; }
  if (bitRead(mask,4)){ ledsc[4]=CRGB::Blue; }
  if (bitRead(mask,3)){ ledsc[8]=CRGB::Blue; }
  if (bitRead(mask,2)){ ledsc[12]=CRGB::Blue; }
  if (bitRead(mask,1)){ ledsc[16]=CRGB::Blue; }
  if (bitRead(mask,0)){ ledsc[20]=CRGB::Blue; }
  
}

void rightvu(){
  static unsigned long timeLast=0;
  unsigned long elapsed=millis();
  if ((elapsed - timeLast) < RIGHTVUSPEED) return;
  timeLast = elapsed;

  ledsc[1] = CRGB::Black;
  ledsc[2] = CRGB::Black;
  ledsc[3] = CRGB::Black;
  ledsc[5] = CRGB::Black;
  ledsc[6] = CRGB::Black;
  ledsc[7] = CRGB::Black;
  ledsc[9] = CRGB::Black;
  ledsc[10] = CRGB::Black;
  ledsc[11] = CRGB::Black;
  ledsc[13] = CRGB::Black;
  ledsc[14] = CRGB::Black;
  ledsc[15] = CRGB::Black;
  ledsc[17] = CRGB::Black;
  ledsc[18] = CRGB::Black;
  ledsc[19] = CRGB::Black;
  ledsc[21] = CRGB::Black;
  ledsc[22] = CRGB::Black;
  ledsc[23] = CRGB::Black;

  int diff = random(0,2);
  if (diff == 0){
    if (bar>0){
      bar--;
    }
  }
  else {
    if (bar<6){
      bar++;
    }
  }
  
  if (bar==6){
    ledsc[1] = CRGB::Red;
    ledsc[2] = CRGB::Red;
    ledsc[3] = CRGB::Red;
  }
  
  if (bar>=5){
    ledsc[5] = CRGB::Red;
    ledsc[6] = CRGB::Red;
    ledsc[7] = CRGB::Red;
  }

  if (bar>=4){
    ledsc[9] = CRGB::Yellow;
    ledsc[10] = CRGB::Yellow;
    ledsc[11] = CRGB::Yellow;
  }
  
  if (bar>=3){
    ledsc[13] = CRGB::Yellow;
    ledsc[14] = CRGB::Yellow;
    ledsc[15] = CRGB::Yellow;
  }
  
  if (bar>=2){
    ledsc[17] = CRGB::Green;
    ledsc[18] = CRGB::Green;
    ledsc[19] = CRGB::Green;
  }
  
  if (bar>=1){
    ledsc[21] = CRGB::Green;
    ledsc[22] = CRGB::Green;
    ledsc[23] = CRGB::Green;
  }
}

void red_big() {
  //D2 7+8 (index 6+7)

  static unsigned long timeLast=0;
  unsigned long elapsed=millis();
  if ((elapsed - timeLast) < REDBOTTOMLEDSPEED) return;
  timeLast = elapsed;
  
  ledsa[6] = CRGB::Black;
  ledsa[7] = CRGB::Black;

  if ( random(0,2) ){ ledsa[6] = CRGB::Red; }
  if ( random(0,2) ){ ledsa[7] = CRGB::Red; }
}

void white_bottom() {
  //D2 1-6 (index 0-5)

  static unsigned long timeLast=0;
  unsigned long elapsed=millis();
  if ((elapsed - timeLast) < BOTTOMLEDSPEED) return;
  timeLast = elapsed;
  
  ledsa[0] = CRGB::Black;
  ledsa[1] = CRGB::Black;
  ledsa[2] = CRGB::Black;
  ledsa[3] = CRGB::Black;
  ledsa[4] = CRGB::Black;
  ledsa[5] = CRGB::Black;

  int mask = random(0,64);
  
  if (bitRead(mask,0)) { ledsa[0] = CRGB::White; }
  if (bitRead(mask,1)) { ledsa[1] = CRGB::White; }
  if (bitRead(mask,2)) { ledsa[2] = CRGB::White; }
  if (bitRead(mask,3)) { ledsa[3] = CRGB::White; }
  if (bitRead(mask,4)) { ledsa[4] = CRGB::White; }
  if (bitRead(mask,5)) { ledsa[5] = CRGB::White; }
}

void top_yellow_green() {
  //D11 1-9 (index 0-8) yellow
  //D11 10-18 (index 9-17) green

  static unsigned long timeLast=0;
  unsigned long elapsed=millis();
  if ((elapsed - timeLast) < TOPLEDSPEED) return;
  timeLast = elapsed;

  for(int i=0;i<18;i++){
    ledsd[i] = CRGB::Black;
  }

  int mask = random(0,64);

  if (bitRead(mask,0)){
    ledsd[0] = CRGB::Yellow;
    ledsd[3] = CRGB::Yellow;
  }
  if (bitRead(mask,1)){
    ledsd[3] = CRGB::Yellow;
    ledsd[6] = CRGB::Yellow;
  }
  if (bitRead(mask,2)){
    ledsd[1] = CRGB::Yellow;
    ledsd[4] = CRGB::Yellow;
  }
  if (bitRead(mask,3)){
    ledsd[4] = CRGB::Yellow;
    ledsd[7] = CRGB::Yellow;
  }
  if (bitRead(mask,4)){
    ledsd[2] = CRGB::Yellow;
    ledsd[5] = CRGB::Yellow;
  }
  if (bitRead(mask,5)){
    ledsd[5] = CRGB::Yellow;
    ledsd[8] = CRGB::Yellow;
  }

  int mask2 = random(0,64);

  if (bitRead(mask2,0)){
    ledsd[9] = CRGB::Green;
    ledsd[12] = CRGB::Green;
  }
  if (bitRead(mask2,1)){
    ledsd[12] = CRGB::Green;
    ledsd[15] = CRGB::Green;
  }
  if (bitRead(mask2,2)){
    ledsd[10] = CRGB::Green;
    ledsd[13] = CRGB::Green;
  }
  if (bitRead(mask2,3)){
    ledsd[13] = CRGB::Green;
    ledsd[16] = CRGB::Green;
  }
  if (bitRead(mask2,4)){
    ledsd[11] = CRGB::Green;
    ledsd[14] = CRGB::Green;
  }
  if (bitRead(mask2,5)){
    ledsd[14] = CRGB::Green;
    ledsd[17] = CRGB::Green;
  }

}

void vu(){
  uint8_t beat = beatsin8( 62, 14, 255);
 
  for(int i=0;i<NUM_LEDSB/2;i++){
    int x = map(beat,0,255,0,14);
    if (i<6) {
      if (x>i) {
        ledsb[i] = CRGB::Green;
        ledsb[i+NUM_LEDSB/2] = CRGB::Green;
      }
      else {
        ledsb[i] = CRGB::Black;
        ledsb[i+NUM_LEDSB/2] = CRGB::Black;
      }
    }
    if (i>=6 && i<11){
      if (x>=i){
        ledsb[i] = CRGB::Yellow;
        ledsb[i+NUM_LEDSB/2] = CRGB::Yellow;
      }
      else {
        ledsb[i] = CRGB::Black;
        ledsb[i+NUM_LEDSB/2] = CRGB::Black;
      }
    }
    if (i>=11){
      if (x>=i){
      ledsb[i] = CRGB::Red;
      ledsb[i+NUM_LEDSB/2] = CRGB::Red;
      }
      else {
      ledsb[i] = CRGB::Black;
      ledsb[i+NUM_LEDSB/2] = CRGB::Black;
      }
    }
  }
}

// CBI Code


void updateCBILEDs()
{
  static unsigned long timeLast=0;
  unsigned long elapsed;
  elapsed=millis();

  if ((elapsed - timeLast) < CBISPEED_RANDOM) return;

  timeLast = elapsed; 

  //FastLeds
  if (effektcbi == 0){
    for (int a=0; a < 20;a=a+5){
      byte ra = random(255);
      for(int b=0; b<5;b++){
        if bitRead(ra,b) {
          ledsf[b+a] = CRGB::Black;
        }
        else {
          ledsf[b+a] = CRGB::Red;
        }
      }
    }
  }
  if (effektcbi == 1){
    fill_rainbow( ledsf, 20, gHue, 7);
  }
  if (effektcbi == 2) {
    fadeToBlackBy( ledsf, 20, 10);
    int pos = random16(20);
    ledsf[pos] += CHSV( gHue + random8(64), 200, 255);
  }
  #ifndef monitorVcc
    ledsf[20] = CRGB::Black;
    ledsf[21] = CRGB::Black;
    ledsf[22] = CRGB::Black;
    byte vc = random(32);
    if bitRead(vc,0){ 
      ledsf[20] = CRGB::Green;
    }
    if bitRead(vc,1){ 
      ledsf[21] = CRGB::Yellow;
    }
    if bitRead(vc,2){ 
      ledsf[22] = CRGB::Red;
    }
  #endif

}

void getVCC()
{
  value = analogRead(analoginput); // this must be between 0.0 and 5.0 - otherwise you'll let the blue smoke out of your arduino
  vout= (value * 5.0)/1024.0;  //voltage coming out of the voltage divider
  vin = vout / (R2/(R1+R2)); //voltage to display

  if (vin >= redVCC){
      ledsf[20] = CRGB::Green;
  }
  if (vin >= yellowVCC){
      ledsf[21] = CRGB::Yellow;
  }
  if (vin >= greenVCC){
      ledsf[22] = CRGB::Red;
  }
  
  //Serial.print("Volt Out = ");                                  // DEBUG CODE
  //Serial.print(vout, 1);   //Print float "vin" with 1 decimal   // DEBUG CODE
  //Serial.print("\tVolts Calc = ");                             // DEBUG CODE
  //Serial.println(vin, 1);   //Print float "vin" with 1 decimal   // DEBUG CODE
}
